"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.completer = exports.wrapField = exports.createCompletionFilter = void 0;
const semver_1 = require("semver");
const mongodb_constants_1 = require("@mongodb-js/mongodb-constants");
const ALL_COMPLETIONS = [
    ...mongodb_constants_1.ACCUMULATORS,
    ...mongodb_constants_1.BSON_TYPES,
    ...mongodb_constants_1.BSON_TYPE_ALIASES,
    ...mongodb_constants_1.CONVERSION_OPERATORS,
    ...mongodb_constants_1.EXPRESSION_OPERATORS,
    ...mongodb_constants_1.JSON_SCHEMA,
    ...mongodb_constants_1.QUERY_OPERATORS,
    ...mongodb_constants_1.STAGE_OPERATORS,
];
function matchesMeta(filter, meta) {
    const metaParts = meta.split(':');
    return filter.some((metaFilter) => {
        const filterParts = metaFilter.split(':');
        return (filterParts.length === metaParts.length &&
            filterParts.every((part, index) => {
                return part === '*' || part === metaParts[index];
            }));
    });
}
function createCompletionFilter(prefix, serverVersion, filterMeta) {
    var _a, _b, _c;
    const currentServerVersion = (_c = (_b = (_a = serverVersion.match(/^(?<version>\d+?\.\d+?\.\d+?)/)) === null || _a === void 0 ? void 0 : _a.groups) === null || _b === void 0 ? void 0 : _b.version) !== null && _c !== void 0 ? _c : serverVersion;
    return ({ value, version: minServerVersion, meta }) => {
        return (value.toLowerCase().startsWith(prefix.toLowerCase()) &&
            (0, semver_1.gte)(currentServerVersion, minServerVersion) &&
            (!filterMeta || matchesMeta(filterMeta, meta)));
    };
}
exports.createCompletionFilter = createCompletionFilter;
function isValidIdentifier(identifier) {
    if (/[.\s"'()[\];={}]/.test(identifier)) {
        return false;
    }
    try {
        new Function(`"use strict";let ${identifier};`);
        return true;
    }
    catch (_a) {
        return false;
    }
}
function wrapField(field) {
    return isValidIdentifier(field)
        ? field
        : `"${field.replace(/["\\]/g, '\\$&')}"`;
}
exports.wrapField = wrapField;
function completer(prefix = '', options = {}, completions = ALL_COMPLETIONS) {
    const { serverVersion = '999.999.999', fields = [], meta } = options;
    const completionsFilter = createCompletionFilter(prefix, serverVersion, meta);
    const completionsWithFields = [].concat(completions, fields.flatMap((field) => {
        return [
            { value: field, meta: 'field:identifier', version: '0.0.0' },
            { value: `$${field}`, meta: 'field:reference', version: '0.0.0' },
        ];
    }));
    return completionsWithFields
        .filter((completion) => {
        return completionsFilter(completion);
    })
        .map((completion) => {
        var _a;
        return {
            value: completion.value,
            meta: completion.meta,
            score: (_a = completion.score) !== null && _a !== void 0 ? _a : 1,
            ...(completion.description && { description: completion.description }),
            ...(completion.snippet && { snippet: completion.snippet }),
        };
    });
}
exports.completer = completer;
//# sourceMappingURL=autocompleter.js.map